/**
 *  @brief example code for liboauth using http://term.ie/oauth/example
 *  @file oauthexample.c
 *  @author Robin Gareus <robin@gareus.org>
 *
 * Copyright 2008 Robin Gareus <robin@gareus.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "oauth.h"

#include <vector>
#include <string>

#include <shlobj.h>

/**
 * split and parse URL parameters replied by the test-server
 * into <em>oauth_token</em> and <em>oauth_token_secret</em>.
 */
bool parse_reply(const char *reply, std::string *token, std::string *secret)
{
	std::vector<std::string> arr;

	char const *end = reply + strlen(reply);
	char const *left = reply;
	char const *right = left;
	while (1) {
		int c = 0;
		if (right < end) {
			c = *right;
		}
		if (c == 0 || c == '&') {
			std::string str(left, right);
			arr.push_back(str);
			if (c == 0) {
				break;
			}
			right++;
			left = right;
		}
		right++;
	}

	char const *oauth_token = 0;
	char const *oauth_token_secret = 0;

	for (std::vector<std::string>::const_iterator it = arr.begin(); it != arr.end(); it++) {
		if (strncmp(it->c_str(), "oauth_token=", 12) == 0) {
			oauth_token = it->c_str() + 12;
		} else if (strncmp(it->c_str(), "oauth_token_secret=", 19) == 0) {
			oauth_token_secret = it->c_str() + 19;
		}
	}

	if (oauth_token && oauth_token_secret) {
		if (token) {
			*token = oauth_token;
		}
		if (secret) {
			*secret = oauth_token_secret;
		}
		return true;
	}

	return false;
}

#include <stdio.h>

std::string inputtext()
{
	char tmp[100];
	fgets(tmp, 100, stdin);
	size_t i = strlen(tmp);
	while (i > 0 && (tmp[i - 1] == '\n' || tmp[i - 1] == '\r')) {
		i--;
	}
	return std::string(tmp, tmp + i);
}

//static char const consumer_key[] = "xxxxxxxxxxxxxxxxxxxxxx";
//static char const consumer_secret[] = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
#include "../twitter.soramimi_jp.txt" // ̃t@C͔閧łB̗lȓeꂽ̃t@CłB


#include "socket.h"

void twitter_example()
{
	static const char *request_token_uri = "http://api.twitter.com/oauth/request_token";
	static const char *authorize_uri = "http://api.twitter.com/oauth/authorize";
	static const char *access_token_uri = "http://api.twitter.com/oauth/access_token";

	std::string c_key; //< consumer key
	std::string c_secret; //< consumer secret

	std::string t_key;
	std::string t_secret;

	std::string req_url;
	std::string postarg;
	std::string reply;

#if 1

	//

	c_key = consumer_key;
	c_secret = consumer_secret;

	//

	printf("Request token..\n");
	
	{
		std::string reply;
		std::string req_url = oauth_sign_url2(request_token_uri, &postarg, OA_HMAC, 0, c_key.c_str(), c_secret.c_str(), 0, 0);
		reply = oauth_http_post(req_url.c_str(),postarg.c_str(), false);

		if (!parse_reply(reply.c_str(), &t_key, &t_secret)) {
			throw "failed to get request token.";
		}
	}

	//

	printf("Authorize..\n");

	{
		std::string req_url = oauth_sign_url2(authorize_uri, 0, OA_HMAC, 0, c_key.c_str(), c_secret.c_str(), t_key.c_str(), t_secret.c_str());

		printf("Opening...\n");
		puts(req_url.c_str());

		ShellExecute(0, 0, req_url.c_str(), 0, 0, SW_SHOW); // EFuuEUN
	}

	//

	{
		printf("Input PIN: ");
		std::string pin = inputtext();
		putchar('\n');

		printf("Access token..\n");

		std::string url = access_token_uri;
		url += "?oauth_verifier=";
		url += pin;

		std::string req_url = oauth_sign_url2(url.c_str(), 0, OA_HMAC, 0, c_key.c_str(), 0, t_key.c_str(), 0);
		std::string reply = oauth_http_get(req_url.c_str(), postarg.c_str());

		if (!parse_reply(reply.c_str(), &t_key, &t_secret)) {
			throw "failed to get access token.";
		}
	}

	// now retrieved 't_key' is access token and 't_secret' is access secret.

	printf("access key: %s\n", t_key.c_str());
	printf("access secret: %s\n", t_secret.c_str());
#else
    c_key = "xxxxxxxxxxxxxxxxxxxxxx";
    c_secret = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
    t_key = "00000000-xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
    t_secret = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
#endif
	// call Twitter API

	printf("make some request..\n");

	printf("input message: ");
	std::string message = inputtext();
	putchar('\n');
	if (!message.empty()) {
		{
			int n;
			wchar_t ucs2[1000];
			char utf8[1000];
			n = MultiByteToWideChar(CP_ACP, 0, message.c_str(), message.size(), ucs2, 1000);
			n = WideCharToMultiByte(CP_UTF8, 0, ucs2, n, utf8, 1000, 0, 0);
			message.assign(utf8, n);
		}

//		std::string uri = "http://api.twitter.com/statuses/update.xml";
//		std::string uri = "http://api.twitter.com/1/statuses/update.xml"; // 2012-11-15
		std::string uri = "http://api.twitter.com/1.1/statuses/update.json"; // 2013-02-26
		uri += "?status=";
		uri += oauth_url_escape(message.c_str());

		req_url = oauth_sign_url2(uri.c_str(), &postarg, OA_HMAC, 0, c_key.c_str(), c_secret.c_str(), t_key.c_str(), t_secret.c_str());
		reply = oauth_http_post(req_url.c_str(), postarg.c_str(), false);
	}
}

int main (int argc, char **argv)
{
	try {
		Socket::initialize();
		twitter_example();
	} catch (char const *error) {
		puts(error);
	}
	return 0;
}
